/*
OpenLaserShowController.h : main header file for the OpenLaserShowController DLL

Copyright (c) 2009 Chris Favreau

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef __OPENLASERSHOWCONTROLLER_H__
#define __OPENLASERSHOWCONTROLLER_H__

// ****************************************************************************************************************
// ** Constants
// ****************************************************************************************************************

// Shutter State
#define LASER_SHOW_DEVICE_SHUTTER_STATE_ON		0
#define LASER_SHOW_DEVICE_SHUTTER_STATE_OFF		1

// Errors
// TODO - add errors to this list so we can be more verbose in describing the particular error that occurred.
#define OLSC_ERROR_SUCCESS						1
#define OLSC_ERROR_NONE							0
#define OLSC_ERROR_INVALID_PARAMETER			-1
#define OLSC_ERROR_FAILED						-2

// ****************************************************************************************************************
// ** Callback Functions
// ****************************************************************************************************************

typedef void (*POLSCCALLBACK)(int device, DWORD status);

// ****************************************************************************************************************
// ** Data Structures
// ****************************************************************************************************************

struct LASER_SHOW_DEVICE_CAPABILITIES
{
	// Device Name
	char name[1024];
	// Device Version
	int	version_major;
	int	version_minor;
	// Min Speed/Max Speed
	int min_speed;
	int max_speed;
	// Min Frame Size/Max Frame Size
	int min_frame_size;
	int max_frame_size;
	// DMX In/Out?
	bool has_dmx_in;
	bool has_dmx_out;
	// TTL In/Out?
	bool has_ttl_in;
	bool has_ttl_out;
	// X/Y Resolution (8 to 16 bits)
	int	xy_resolution;
	// Color Resolution (1 to 16 bits)
	int color_resolution;
	// Uses callbacks?
	bool uses_callbacks;
};

struct LASER_SHOW_DEVICE_POINT
{
	WORD x;
	WORD y;
	WORD r;
	WORD g;
	WORD b;
	WORD i;
};

struct LASER_SHOW_DEVICE_FRAME
{
	int display_speed;
	int point_count;
	struct LASER_SHOW_DEVICE_POINT *points;
};

// ****************************************************************************************************************
// ** Function Prototypes
// ****************************************************************************************************************

#define OLSC_EXPORTS

#ifdef OLSC_EXPORTS
#define OLSC_API __declspec(dllexport)
#else
#define OLSC_API __declspec(dllimport)
#endif

#ifdef __cplusplus 
extern "C" {
#endif

OLSC_API int WINAPI OLSC_GetAPIVersion(void);
OLSC_API int WINAPI OLSC_GetInterfaceName(char *pString);
OLSC_API int WINAPI OLSC_Initialize(void);
OLSC_API int WINAPI OLSC_Shutdown(void);
OLSC_API int WINAPI OLSC_GetDeviceCount(void);
OLSC_API int WINAPI OLSC_GetDeviceCapabilities(int device_number, struct LASER_SHOW_DEVICE_CAPABILITIES &device_capabilities);
OLSC_API int WINAPI OLSC_GetLastErrorNumber(int device_number, int &number, char *string_pointer, int string_length);
OLSC_API int WINAPI OLSC_SetConfigFile(char *pFullPathString);									// NEW IN V2
OLSC_API int WINAPI OLSC_ShowConfigWindow(void);												// NEW IN V2
OLSC_API int WINAPI OLSC_Play(int device_number);
OLSC_API int WINAPI OLSC_Pause(int device_number);
OLSC_API int WINAPI OLSC_Shutter(int device_number, int state);
OLSC_API int WINAPI OLSC_WriteFrame(int device_number, struct LASER_SHOW_DEVICE_FRAME frame);
OLSC_API int WINAPI OLSC_WritePoint(int device_number, struct LASER_SHOW_DEVICE_POINT point);	// NEW IN V2
OLSC_API int WINAPI OLSC_WriteBufferedFrame(int device_number);									// NEW IN V2
OLSC_API int WINAPI OLSC_GetStatus(int device_number, DWORD &status);
OLSC_API int WINAPI OLSC_SetCallback(int device_number, POLSCCALLBACK pCallback);				// NEW IN V2
OLSC_API int WINAPI OLSC_WriteDMX(int device_number, int start_address, unsigned char *data_pointer, int length);
OLSC_API int WINAPI OLSC_ReadDMX(int device_number, int start_address, unsigned char *data_pointer, int length);
OLSC_API int WINAPI OLSC_WriteTTL(int device_number, DWORD data);
OLSC_API int WINAPI OLSC_ReadTTL(int device_number, DWORD &data);

#ifdef __cplusplus 
}
#endif

#endif		// __OPENLASERSHOWCONTROLLER_H__

